import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import LinkButton from './LinkButton';

import AnchorTestWrapper from '../../test-utils/AnchorTestWrapper';


const TestLinkButton = LinkButton({
    block: purebem.of('test-button'),
    displayName: 'TestLinkButton'
});

test('should navigate', () => {
    const wrapper = mount(
        <AnchorTestWrapper>
            <TestLinkButton />
        </AnchorTestWrapper>
    );

    wrapper.find(TestLinkButton).simulate('click');

    expect(wrapper.instance().didNavigate).toBe(true);
});

test('should trigger click event when clicked', () => {
    const onClick = jest.fn();
    const button = mount(
        <TestLinkButton
            onClick={ onClick } />
    );

    button.simulate('click');

    expect(onClick).toHaveBeenCalled();
});

test('should not trigger click event nor navigate when clicked and disabled', () => {
    const onClick = jest.fn();
    const wrapper = mount(
        <AnchorTestWrapper>
            <TestLinkButton
                disabled={ true }
                onClick={ onClick } />
        </AnchorTestWrapper>
    );

    wrapper.find(TestLinkButton).simulate('click');

    expect(onClick).not.toHaveBeenCalled();
    expect(wrapper.instance().didNavigate).toBe(false);
});
