import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import RouteButton from './RouteButton';

import AnchorTestWrapper from '../../test-utils/AnchorTestWrapper';


const TestRouteButton = RouteButton({
    block: purebem.of('test-button'),
    displayName: 'TestRouteButton'
});

test('should trigger click event when clicked', () => {
    let value = false;
    const button = mount(
        <TestRouteButton
            onClick={ (evt) => {
                evt.preventDefault(); // prevent react-router from blowing up.
                value = true;
            } } />
    );

    button.simulate('click');

    expect(value).toBe(true);
});

test('should not trigger click event nor navigate when clicked and disabled', () => {
    const onClick = jest.fn();
    const wrapper = mount(
        <AnchorTestWrapper>
            <TestRouteButton
                disabled={ true }
                onClick={ onClick } />
        </AnchorTestWrapper>
    );

    wrapper.find(TestRouteButton).simulate('click');

    expect(onClick).not.toHaveBeenCalled();
    expect(wrapper.didNavigate).toBeFalsy();
});
