import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import simulateLabelClick from '../../test-utils/simulateLabelClick';

import Checkbox from './Checkbox';


const TestCheckbox = Checkbox({
    block: purebem.of('test-checkbox'),
    displayName: 'TestCheckbox'
});

test('should be checked when clicked and is unchecked', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            onChange={ onChange }
            value={ false } />
    );

    simulateLabelClick(checkbox);

    expect(onChange).toHaveBeenCalledWith(true);
});

test('should be unchecked when clicked and is checked', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            onChange={ onChange }
            value={ true } />
    );

    simulateLabelClick(checkbox);

    expect(onChange).toHaveBeenCalledWith(false);
});

test('should remain unchecked when disabled and clicked', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            disabled={ true }
            onChange={ onChange }
            value={ false } />
    );

    simulateLabelClick(checkbox);

    expect(onChange).not.toHaveBeenCalled();
});

test('should remain checked when disabled and clicked', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            disabled={ true }
            onChange={ onChange }
            value={ true } />
    );

    simulateLabelClick(checkbox);

    expect(onChange).not.toHaveBeenCalled();
});

test('should be checked when using space and is unchecked', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            onChange={ onChange }
            value={ false } />
    );

    checkbox.simulate('keydown', { which: 32 });

    expect(onChange).toHaveBeenCalledWith(true);
});

test('should be unchecked when using space and is checked', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            onChange={ onChange }
            value={ true } />
    );

    checkbox.simulate('keydown', { which: 32 });

    expect(onChange).toHaveBeenCalledWith(false);
});

test('should remain unchecked when disabled and using space', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            disabled={ true }
            onChange={ onChange }
            value={ false } />
    );

    checkbox.simulate('keydown', { which: 32 });

    expect(onChange).not.toHaveBeenCalled();
});

test('should remain checked when disabled and using space', () => {
    const onChange = jest.fn();
    const checkbox = mount(
        <TestCheckbox
            disabled={ true }
            onChange={ onChange }
            value={ true } />
    );

    checkbox.simulate('keydown', { which: 32 });

    expect(onChange).not.toHaveBeenCalled();
});
