import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import isNullOrEmpty from '../../utils/isNullOrEmpty';


export default ({
    block,
    displayName
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            children: PropTypes.node,
            disabled: PropTypes.bool,
            focus: PropTypes.bool,
            onChange: PropTypes.func,
            value: PropTypes.bool
        },

        getDefaultProps() {
            return {
                children: null,
                disabled: false,
                focus: false,
                onChange: (/* value */) => {},
                value: false
            };
        },

        componentDidMount() {
            if (this.props.focus) {
                ReactDOM.findDOMNode(this).focus();
            }
        },

        onChange() {
            this.toggle();
        },

        onKeyDown(evt) {
            switch (evt.which) {
                case 32: // space
                    // don't scroll on page.
                    evt.stopPropagation();
                    evt.preventDefault();

                    this.toggle();
                    break;

                default:
                    break;
            }
        },

        toggle() {
            const { disabled, onChange, value } = this.props;
            if (disabled) {
                return;
            }

            onChange(!value);
        },

        renderContent() {
            return this.props.children
                ? (<span className={ block('content') }>{ this.props.children }</span>)
                : (<span className={ block('no-content') } />);
        },

        render() {
            const { disabled, value } = this.props;
            const { checked, unchecked } = { checked: value, unchecked: !value };

            return (
                <label
                    className={ block({ checked, disabled, unchecked }) }
                    onKeyDown={ this.onKeyDown }
                    tabIndex={ disabled ? null : '0' }>
                    <input
                        type="checkbox"
                        checked={ checked }
                        className={ block('input') }
                        disabled={ disabled }
                        onChange={ this.onChange }
                        onClick={ (evt) => evt.stopPropagation() } />
                    <i
                        className={ block('icon') } />
                    { this.renderContent() }
                </label>
            );
        }

    });

};
