import React from 'react';
import purebem from 'purebem';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import createStore from '../../test-utils/createStore';
import simulateLabelClick from '../../test-utils/simulateLabelClick';

import Checkbox from './Checkbox';
import CheckboxContainer, { changeType } from './CheckboxContainer';
import checkboxChange from './redux/checkboxChange';
import checkboxState from './redux/checkboxState';


const TestCheckbox = Checkbox({
    block: purebem.of('test-checkbox'),
    displayName: 'TestCheckbox'
});

const TestCheckboxContainer = CheckboxContainer({
    Component: TestCheckbox
});

const actionTypePrefix = 'TEST-CHECKBOX';
const path = 'component';
const root = 'test';

test('should dispatch change event on change', () => {
    const store = createStore({
        [root]: {
            [path]: checkboxState()
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestCheckboxContainer
                actionTypePrefix={ actionTypePrefix }
                customActionMetadata={ { index: 1 } }
                path={ path }
                root={ root } />
        </Provider>
    );

    simulateLabelClick(provider);

    const actions = store.getState().actions;

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(checkboxChange({
        custom: { index: 1 },
        path,
        root,
        type: changeType(actionTypePrefix),
        value: true
    }));
});

test('should read value from store', () => {
    const store = createStore({
        [root]: {
            [path]: checkboxState({ value: true })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestCheckboxContainer
                actionTypePrefix={ actionTypePrefix }
                path={ path }
                root={ root } />
        </Provider>
    );

    expect(provider.find(TestCheckbox).prop('value')).toBe(true);
});
