import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import * as lang from '../../utils/lang';
import isNullOrEmpty from '../../utils/isNullOrEmpty';


export default ({
    block,
    displayName
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    // These are extracted for better testability because I prefer to select based on
    // display name instead of CSS class.
    const Lever = React.createClass({

        displayName: `${displayName}Lever`,

        propTypes: {
            disabled: PropTypes.bool.isRequired,
            onClick: PropTypes.func.isRequired,
            onKeyDown: PropTypes.func.isRequired
        },

        render() {
            const { disabled, onClick, onKeyDown } = this.props;
            return (
                <div
                    className={ block('content') }
                    onClick={ onClick }
                    onKeyDown={ onKeyDown }
                    tabIndex={ disabled ? null : '0' }>
                    <div className={ block('content-lever') }></div>
                </div>
            );
        }

    });

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            children: PropTypes.node,
            disabled: PropTypes.bool,
            focus: PropTypes.bool,
            onChange: PropTypes.func,
            value: PropTypes.bool
        },

        getDefaultProps() {
            return {
                children: null,
                disabled: false,
                focus: false,
                onChange: (/* value */) => {},
                value: false
            };
        },

        componentDidMount() {
            if (this.props.focus) {
                ReactDOM.findDOMNode(this.component).focus();
            }
        },

        onChange() {
            this.toggle();
        },

        onKeyDown(evt) {
            switch (evt.which) {
                case 32: // space
                    // don't scroll on page.
                    evt.stopPropagation();
                    evt.preventDefault();

                    this.toggle();
                    break;

                default:
                    break;
            }
        },

        toggle() {
            const { disabled, onChange, value } = this.props;
            if (disabled) {
                return;
            }

            onChange(!value);
        },

        render() {
            const { disabled, value } = this.props;
            const { checked, unchecked } = { checked: value, unchecked: !value };

            return (
                <div className={ block({ checked, disabled, unchecked }) }>
                    <Lever
                        disabled={ disabled }
                        onClick={ this.onChange }
                        onKeyDown={ this.toggle }
                        ref={ (c) => this.component = c } />
                    <div className={ block('label') }>
                        { lang.pgettext('foundation-switch', checked ? 'on' : 'off') }
                    </div>
                </div>
            );
        }

    });

};
