import React from 'react';
import purebem from 'purebem';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import Switch from './Switch';
import SwitchContainer, { changeType } from './SwitchContainer';

import checkboxChange from './redux/checkboxChange';
import checkboxState from './redux/checkboxState';

import createStore from '../../test-utils/createStore';


const TestSwitch = Switch({
    block: purebem.of('test-checkbox'),
    displayName: 'TestSwitch'
});

const TestSwitchContainer = SwitchContainer({
    Component: TestSwitch
});

const actionTypePrefix = 'TEST-CHECKBOX';
const path = 'component';
const root = 'test';

test('should dispatch change event on change', () => {
    const store = createStore({
        [root]: {
            [path]: checkboxState()
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestSwitchContainer
                actionTypePrefix={ actionTypePrefix }
                customActionMetadata={ { index: 1 } }
                path={ path }
                root={ root } />
        </Provider>
    );

    provider.find('TestSwitchLever').simulate('click');

    const actions = store.getState().actions;

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(checkboxChange({
        custom: { index: 1 },
        path,
        root,
        type: changeType(actionTypePrefix),
        value: true
    }));

});

test('should read value from store', () => {
    const store = createStore({
        [root]: {
            [path]: checkboxState({ value: true })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestSwitchContainer
                actionTypePrefix={ actionTypePrefix }
                path={ path }
                root={ root } />
        </Provider>
    );

    expect(provider.find(TestSwitch).prop('value')).toBe(true);
});
