import React from 'react';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import createStore from '../../test-utils/createStore';

import componentMount from './redux/mount';
import componentUnmount from './redux/unmount';
import createComponent from './createComponent';
import createDynamicReducerRegistry from './redux/createDynamicReducerRegistry';


const registry = createDynamicReducerRegistry();
const component = createComponent(registry);

const initialComponentState = { yada: 'yada, yada' };
const initialize = ({ root }) => {
    return {
        type: 'TEST-COMPONENT.INITIALIZE',
        meta: {
            root
        }
    };
};

const root = 'test-component';

const Test = React.createClass({

    render() {
        return (
            <div>Test</div>
        );
    }

});

const TestComponent = component(Test, {
    initialComponentState,
    root
});

const TestComponentWithInitialize = component(Test, {
    initialComponentState,
    initialize,
    root
});


test('should dispatch mount action when mounted', () => {
    const store = createStore();

    mount(
        <Provider store={ store }>
            <TestComponent />
        </Provider>
    );

    const { actions } = store.getState();

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(componentMount({ initialComponentState, root }));
});

test('should dispatch mount and initialze actions when mounted', () => {
    const store = createStore();

    mount(
        <Provider store={ store }>
            <TestComponentWithInitialize />
        </Provider>
    );

    const { actions } = store.getState();

    expect(actions.length).toBe(2);
    expect(actions[0]).toEqual(componentMount({ initialComponentState, root }));
    expect(actions[1]).toEqual(initialize({ root }));
});

test('should dispatch unmount action when unmounted', () => {
    const store = createStore();

    const provider = mount(
        <Provider store={ store }>
            <TestComponent />
        </Provider>
    );

    provider.unmount();

    const { actions } = store.getState();

    expect(actions.length).toBe(2);
    expect(actions[1]).toEqual(componentUnmount({ root }));
});
