import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';
import moment from 'moment';
import purebem from 'purebem';

import * as lang from '../../utils/lang';
import isNullOrEmpty from '../../utils/isNullOrEmpty';
import toTimestamp from '../../utils/toTimestamp';

import getWeeks from './utils/getWeeks';


export default ({
    block,
    displayName
}) => {

    invariant(block !== undefined, 'Missing purebem block');
    invariant(!isNullOrEmpty(displayName), 'Missing display name');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            currentMonth: PropTypes.object,
            hasGrayedOutPast: PropTypes.bool,
            onChange: PropTypes.func,
            onNextMonthClick: PropTypes.func,
            onPreviousMonthClick: PropTypes.func,
            selectedIndex: PropTypes.object,
            value: PropTypes.object
        },

        getDefaultProps() {
            const now = moment();
            return {
                currentMonth: now,
                hasGrayedOutPast: false,
                onChange: (/* currentMonth, selectedIndex, value */) => {},
                onNextMonthClick: () => {},
                onPreviousMonthClick: () => {},
                selectedIndex: now,
                value: null
            };
        },

        renderMonthSelector() {
            const { onPreviousMonthClick, currentMonth, onNextMonthClick } = this.props;
            return (
                <div className={ block('calendar-month-selector') }>
                    <i className={ block('calendar-month-selector-icon', ['left', 'touchable']) } onClick={ onPreviousMonthClick } />
                    <div className={ block('calendar-month-selector-text') }>
                        { lang.pgettext('foundation-datepicker', `Month--${currentMonth.month() + 1}`) } { currentMonth.year() }
                    </div>
                    <i className={ block('calendar-month-selector-icon', ['right', 'touchable']) } onClick={ onNextMonthClick } />
                </div>
            );
        },

        renderWeek(week, index) {
            const { currentMonth, hasGrayedOutPast, onChange, selectedIndex, value } = this.props;
            return (
                <div key={ index } className={ block('calendar-week') }>
                    {
                        week.map((day) => {
                            const now = moment();
                            const date = day.date();

                            const past = hasGrayedOutPast && (day.format('YYYY-MM-DD') < now.format('YYYY-MM-DD'));
                            const outside = day.month() !== currentMonth.month();
                            const selected = value !== null && day.year() === value.year() && day.dayOfYear() === value.dayOfYear();
                            const today = (day.year() === now.year()) && (day.dayOfYear() === now.dayOfYear());
                            const indexed = day.year() === selectedIndex.year() && day.dayOfYear() === selectedIndex.dayOfYear();

                            return (
                                <div key={ date }
                                     className={ block('calendar-day', ['selectable'], { outside, today, past }) }
                                     onClick={ () => onChange(toTimestamp(day.clone().startOf('month')), toTimestamp(day.clone()), toTimestamp(day.clone())) }>
                                    <div className={ block('calendar-day-content', { indexed, selected }) }>{ date }</div>
                                </div>
                            );
                        })
                    }
                </div>
            );
        },

        renderWeeks() {
            const { currentMonth } = this.props;
            const weeks = getWeeks(currentMonth);

            return (
                <div className={ block('calendar-month') }>
                    <div className={ block('calendar-week') }>
                        {
                            ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun'].map((weekday) => {
                                return (
                                    <div key={ weekday } className={ block('calendar-day', ['weekday']) }>
                                        { lang.pgettext('foundation-datepicker', `WeekdayHeader--${weekday}`) }
                                    </div>
                                );
                            })
                        }
                    </div>
                    <hr className={ block('calendar-weekday-separator') } />
                    { weeks.map((week, index) => this.renderWeek(week, index)) }
                </div>
            );
        },

        render() {
            return (
                <div className={ purebem.many(block('calendar', ['expanded']), 'ignore-react-onclickoutside') }>
                    { this.renderMonthSelector() }
                    { this.renderWeeks() }
                </div>
            );
        }

    });

};
