import React from 'react';
import purebem from 'purebem';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import Datepicker from './Datepicker';
import DatepickerContainer, { collapseType, expandType } from './DatepickerContainer';

import datepickerCollapse from './redux/datepickerCollapse';
import datepickerExpand from './redux/datepickerExpand';
import datepickerState from './redux/datepickerState';

import createStore from '../../test-utils/createStore';


const TestDatepicker = Datepicker({
    block: purebem.of('test-datepicker'),
    displayName: 'TestDatepicker'
});

const TestDatepickerContainer = DatepickerContainer({
    Component: TestDatepicker
});

const actionTypePrefix = 'TEST-DATEPICKER';
const path = 'component';
const root = 'test';

test('should dispatch expand event when clicked and collapsed', () => {
    const store = createStore({
        [root]: {
            [path]: datepickerState({
                value: +new Date(2000, 0, 1)
            })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestDatepickerContainer
                actionTypePrefix={ actionTypePrefix }
                customActionMetadata={ { index: 1 } }
                path={ path }
                root={ root } />
        </Provider>
    );

    provider.find('TestDatepickerControl').simulate('click');

    const actions = store.getState().actions;

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(datepickerExpand({
        custom: { index: 1 },
        path,
        root,
        type: expandType(actionTypePrefix)
    }));
});

test('should dispatch collapse event when clicked and expanded', () => {
    const store = createStore({
        [root]: {
            [path]: datepickerState({
                expanded: true,
                value: +new Date(2000, 0, 1)
            })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestDatepickerContainer
                actionTypePrefix={ actionTypePrefix }
                customActionMetadata={ { index: 1 } }
                path={ path }
                root={ root } />
        </Provider>
    );

    provider.find('TestDatepickerControl').simulate('click');

    const actions = store.getState().actions;

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(datepickerCollapse({
        currentMonth: +new Date(2000, 0, 1),
        custom: { index: 1 },
        path,
        root,
        selectedIndex: +new Date(2000, 0, 1),
        type: collapseType(actionTypePrefix)
    }));
});

test('should read values from store', () => {
    const store = createStore({
        [root]: {
            [path]: datepickerState({
                expanded: true,
                value: +new Date(2000, 0, 1)
            })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestDatepickerContainer
                actionTypePrefix={ actionTypePrefix }
                path={ path }
                root={ root } />
        </Provider>
    );

    const component = provider.find(TestDatepicker);
    expect(component.prop('currentMonth')).toBe(+new Date(2000, 0, 1));
    expect(component.prop('expanded')).toBe(true);
    expect(component.prop('selectedIndex')).toBe(+new Date(2000, 0, 1));
    expect(component.prop('value')).toBe(+new Date(2000, 0, 1));
});
