import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import isHandheld from '../../utils/isHandheld';
import isNullOrEmpty from '../../utils/isNullOrEmpty';


export default ({
    block,
    displayName
}) => {

    invariant(block !== undefined, 'Missing purebem block');
    invariant(!isNullOrEmpty(displayName), 'Missing display name');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            defaultValue: PropTypes.string,
            disabled: PropTypes.bool,
            onClick: PropTypes.func,
            onTouchableBlur: PropTypes.func,
            onTouchableChange: PropTypes.func,
            onTouchableFocus: PropTypes.func,
            value: PropTypes.string
        },

        getDefaultProps() {
            return {
                defaultValue: null,
                onClick: () => {},
                onTouchableBlur: () => {},
                onTouchableChange: (/* evt */) => {},
                onTouchableFocus: () => {},
                value: null
            };
        },

        renderHandheldControl() {
            if (!isHandheld()) {
                return null;
            }

            const { defaultValue, disabled, onTouchableBlur, onTouchableChange, onTouchableFocus } = this.props;
            return (
                <input
                    defaultValue={ defaultValue }
                    disabled={ disabled }
                    className={ block('control-input-date') }
                    onChange={ onTouchableChange }
                    onBlur={ onTouchableBlur }
                    onFocus={ onTouchableFocus }
                    type="date" />
            );
        },

        render() {
            const { onClick, value } = this.props;
            return (
                <div
                    className={ block('control') }
                    onClick={ onClick }>
                    { this.renderHandheldControl() }
                    <i className={ block('control-icon') } />
                    <div className={ block('control-text') }>{ value }</div>
                </div>
            );
        }
    });

};
