import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import datepickerState from './redux/datepickerState';


export default ({
    Component
} = {}) => {

    invariant(Component !== undefined, 'Missing Datepicker component.');

    return React.createClass({

        displayName: `${Component.displayName}`,

        mixins: [PureRenderMixin],

        propTypes: {
            onChange: PropTypes.func,
            initialValue: PropTypes.oneOfType([
                PropTypes.instanceOf(Date),
                PropTypes.number
            ])
        },

        getDefaultProps() {
            return {
                onChange: (/* value */) => {},
                value: null
            };
        },

        getInitialState() {
            const { initialValue } = this.props;
            const { currentMonth, expanded, selectedIndex, value } = datepickerState({ value: initialValue });

            return {
                currentMonth,
                expanded,
                selectedIndex,
                value
            };
        },

        onChange(currentMonth, selectedIndex, value) {
            this.setState({ currentMonth, expanded: false, selectedIndex, value },
                () => this.props.onChange(value));
        },

        onCollapse(currentMonth, selectedIndex) {
            this.setState({ currentMonth, expanded: false, selectedIndex });
        },

        onExpand() {
            this.setState({ expanded: true });
        },

        onSelectedIndexChange(currentMonth, selectedIndex) {
            this.setState({ currentMonth, selectedIndex });
        },

        render() {
            return (
                <Component
                    { ...this.props }
                    { ...this.state }
                    onChange={ this.onChange }
                    onCollapse={ this.onCollapse }
                    onExpand={ this.onExpand }
                    onSelectedIndexChange={ this.onSelectedIndexChange } />
            );
        }

    });

};
