import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import isNullOrEmpty from '../../utils/isNullOrEmpty';


export default ({
    block,
    displayName,
    uiPropDefaultType = PropTypes.string,
    uiPropDefaultValue = '',
    uiPropGetValue = (ui) => ui
}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            onHide: PropTypes.func,
            text: PropTypes.string,
            timeoutId: PropTypes.number,
            timestamp: PropTypes.number,
            ui: uiPropDefaultType
        },

        getDefaultProps() {
            return {
                onHide: (/* { timeoutId, timestamp } */) => {},
                text: '',
                timeoutId: 0,
                timestamp: 0,
                ui: uiPropDefaultValue
            };
        },

        render() {
            const { onHide, text, timeoutId, timestamp, ui } = this.props;
            return (
                <div className={ block({ ui: uiPropGetValue(ui) }) }>
                    <div className={ block('text') }>{ text }</div>
                    <i className={ block('close') } onClick={ () => onHide({ timeoutId, timestamp }) } />
                </div>
            );
        }

    });

};
