import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import Tether from 'tether';
import throttle from 'lodash.throttle';

import isInViewport from '../../utils/isInViewport';


const FoundationTether = React.createClass({

    propTypes: {
        children: PropTypes.node,
        target: PropTypes.func,
        tetherTo: PropTypes.string
    },

    getDefaultProps() {
        return {
            children: null,
            target: () => {},
            tetherTo: 'content'
        };
    },

    componentWillMount() {
        const element = document.getElementById(this.props.tetherTo);
        // this is mainly to support testing, should probably warn in browser if condition is met.
        if (element === null) {
            return;
        }

        this.tetherElement = document.createElement('div');
        this.tetherElement.className = 'foundation-tether';
        element.appendChild(this.tetherElement);

        this.onResize = throttle(this.setOrientation, 250);
    },

    componentDidMount() {
        if (!this.tetherElement) {
            return;
        }

        this.renderChildren();

        this.tether = new Tether(this.getOptions());
        this.setOrientation();

        window.addEventListener('resize', this.onResize);
    },

    componentDidUpdate() {
        this.renderChildren();
    },

    componentWillUnmount() {
        if (!this.tetherElement) {
            return;
        }

        window.removeEventListener('resize', this.onResize);

        this.tether.destroy();

        ReactDOM.unmountComponentAtNode(this.tetherElement);

        this.tetherElement.parentNode.removeChild(this.tetherElement);
        this.tetherElement = null;

        this.child = null;
    },

    getOptions(overrides = {}) {
        return {
            element: this.tetherElement,
            target: this.props.target(),
            attachment: 'top left',
            targetAttachment: 'bottom left',
            ...overrides
        };
    },

    setOrientation() {
        // try right orientation.
        this.tether.setOptions(this.getOptions({
            attachment: 'top left',
            targetAttachment: 'bottom left'
        }));

        const node = ReactDOM.findDOMNode(this.child);
        if (!isInViewport(node)) {
            // try left orientation.
            this.tether.setOptions(this.getOptions({
                attachment: 'top right',
                targetAttachment: 'bottom right'
            }));

            if (!isInViewport(node)) {
                // try middle orientation.
                this.tether.setOptions(this.getOptions({
                    attachment: 'top middle',
                    targetAttachment: 'bottom middle'
                }));
            }
        }
    },

    renderChildren() {
        if (!this.tetherElement) {
            return;
        }

        this.child = ReactDOM.render(this.props.children, this.tetherElement);
    },

    render() {
        return (<div></div>);
    }

});

export default FoundationTether;
