import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import Monthpicker from './Monthpicker';
import monthpickerState from './redux/monthpickerState';


const TestMonthpicker = Monthpicker({
    block: purebem.of('test-monthpicker'),
    displayName: 'TestMonthpicker'
});

test('should render year, and month (timestamp)', () => {
    const monthpicker = mount(
        <TestMonthpicker
            { ...monthpickerState({ value: +new Date(2000, 0, 1) }) } />
    );

    expect(monthpicker.text()).toBe('Month--1 2000');
});

test('should render year, and month (Date)', () => {
    const monthpicker = mount(
        <TestMonthpicker
            { ...monthpickerState({ value: new Date(2000, 0, 1) }) } />
    );

    expect(monthpicker.text()).toBe('Month--1 2000');
});

test('should render year, and month range with negative offset', () => {
    const monthpicker = mount(
        <TestMonthpicker
            yearOffset={ -1 }
            { ...monthpickerState({ value: +new Date(2000, 0, 1) }) } />
    );

    expect(monthpicker.text()).toBe('Month-abbrv--1 1999 － Month-abbrv--1 2000');
});

test('should render year, and month range with offset', () => {
    const monthpicker = mount(
        <TestMonthpicker
            yearOffset={ 1 }
            { ...monthpickerState({ value: +new Date(2000, 0, 1) }) } />
    );

    expect(monthpicker.text()).toBe('Month-abbrv--1 2000 － Month-abbrv--1 2001');
});

test('should change value when clicking next', () => {
    const onChange = jest.fn();
    const monthpicker = mount(
        <TestMonthpicker
            onChange={ onChange }
            { ...monthpickerState({ value: +new Date(2000, 0, 1) }) } />
    );

    monthpicker.find('TestMonthpickerNextMonth').simulate('click');
    expect(onChange).toHaveBeenCalledWith(+new Date(2000, 1, 1));
});

test('should change value when clicking previous', () => {
    const onChange = jest.fn();
    const monthpicker = mount(
        <TestMonthpicker
            onChange={ onChange }
            { ...monthpickerState({ value: +new Date(2000, 0, 1) }) } />
    );

    monthpicker.find('TestMonthpickerPreviousMonth').simulate('click');
    expect(onChange).toHaveBeenCalledWith(+new Date(1999, 11, 1));
});
