import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import moment from 'moment';

import * as lang from '../../utils/lang';
import toTimestamp from '../../utils/toTimestamp';


export default ({
    block,
    displayName
} = {}) => {

    // These are extracted for better testability because I prefer to select based on
    // display name instead of CSS class.
    const NextMonth = React.createClass({

        displayName: `${displayName}NextMonth`,

        propTypes: {
            onClick: PropTypes.func.isRequired
        },

        render() {
            return (
                <i className={ block('selector-icon', ['next', 'touchable']) } onClick={ this.props.onClick } />
            );
        }

    });

    const PreviousMonth = React.createClass({

        displayName: `${displayName}PreviousMonth`,

        propTypes: {
            onClick: PropTypes.func.isRequired
        },

        render() {
            return (
                <i className={ block('selector-icon', ['previous', 'touchable']) } onClick={ this.props.onClick } />
            );
        }

    });

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            onChange: PropTypes.func,
            yearOffset: PropTypes.number,
            value: PropTypes.oneOfType([
                PropTypes.instanceOf(Date),
                PropTypes.number
            ])
        },

        getDefaultProps() {
            return {
                onChange: () => {},
                value: +new Date(),
                yearOffset: 0
            };
        },

        onNextMonth() {
            const { onChange, value } = this.props;
            onChange(toTimestamp(moment(value).add(1, 'months')));
        },

        onPreviousMonth() {
            const { onChange, value } = this.props;
            onChange(toTimestamp(moment(value).subtract(1, 'months')));
        },

        renderContent() {
            const { value, yearOffset } = this.props;
            const date = moment(value);

            switch (true) {
                case yearOffset === 0:
                    return (
                        <span>{ lang.pgettext('foundation-monthpicker', `Month--${date.month() + 1}`) } { date.year() }</span>
                    );

                case yearOffset < 0:
                    return (
                        <span>{ lang.pgettext('foundation-monthpicker', `Month-abbrv--${date.month() + 1}`) } { date.year() - 1 } &#65293; { lang.pgettext('foundation-monthpicker', `Month-abbrv--${date.month() + 1}`) } { date.year() }</span>
                    );

                case yearOffset > 0:
                    return (
                        <span>{ lang.pgettext('foundation-monthpicker', `Month-abbrv--${date.month() + 1}`) } { date.year() } &#65293; { lang.pgettext('foundation-monthpicker', `Month-abbrv--${date.month() + 1}`) } { date.year() + 1 }</span>
                    );
            }
        },

        render() {
            return (
                <div className={ block() }>
                    <PreviousMonth
                        onClick={ this.onPreviousMonth } />
                    <div className={ block('selector-text') }>
                        { this.renderContent() }
                    </div>
                    <NextMonth
                        onClick={ this.onNextMonth } />
                </div>
            );
        }

    });

};
