import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import RadioButtonGroup from './RadioButtonGroup';
import RadioButtonGroupState from './RadioButtonGroupState';

import radioButtonOptionState from './redux/radioButtonOptionState';


const TestRadioButtonGroup = RadioButtonGroup({
    block: purebem.of('test-checkbox'),
    displayName: 'TestRadioButtonGroup'
});

const TestRadioButtonGroupState = RadioButtonGroupState({
    Component: TestRadioButtonGroup
});

const getOptions = (a = false, b = false, c = false) => {
    return [
        radioButtonOptionState({
            disabled: a,
            text: 'A',
            value: 'a'
        }),
        radioButtonOptionState({
            disabled: b,
            text: 'B',
            value: 'b'
        }),
        radioButtonOptionState({
            disabled: c,
            text: 'C',
            value: 'c'
        })
    ];
};

test('should set value', () => {
    const initialValue = 'a';
    const group = mount(
        <TestRadioButtonGroupState
            initialValue={ initialValue }
            options={ getOptions() } />
    );

    expect(group.state('value')).toBe(initialValue);
});

test('should change value when clicked', () => {
    const onChange = jest.fn();
    const group = mount(
        <TestRadioButtonGroupState
            initialValue={ 'a' }
            onChange={ onChange }
            options={ getOptions() } />
    );

    const options = group.find('TestRadioButtonGroupOption');
    expect(options.length).toBe(3);

    options.at(1).simulate('click');

    expect(onChange).toHaveBeenCalledWith('b');
});
