import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import isNullOrEmpty from '../../utils/isNullOrEmpty';
import Modernizer from '../../utils/Modernizr';


export default ({
    block,
    displayName
}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            autoSize: PropTypes.bool,
            centerScreen: PropTypes.bool,
            delay: PropTypes.number
        },

        getDefaultProps() {
            return {
                centerScreen: false,
                delay: 200
            };
        },

        getInitialState() {
            return {
                active: false
            };
        },

        componentDidMount() {
            this.timer = setTimeout(() => this.setState({ active: true }), this.props.delay);
        },

        componentWillUnmount() {
            clearTimeout(this.timer);
        },

        render() {
            if (!this.state.active) {
                return null;
            }

            const { autoSize, centerScreen } = this.props;
            const animations = Modernizer.cssanimations;
            const fallback = !animations;

            return (
                <div className={ block(['spinner'], { animations, autoSize, centerScreen, fallback }) } />
            );
        }

    });

};
