import React from 'react';
import purebem from 'purebem';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import Textarea from './Textarea';
import TextareaContainer, { changeType } from './TextareaContainer';

import textareaChange from './redux/textareaChange';
import textareaState from './redux/textareaState';

import createStore from '../../test-utils/createStore';


const TestTextarea = Textarea({
    block: purebem.of('test-Textarea'),
    displayName: 'TestTextarea'
});

const TestTextareaContainer = TextareaContainer({
    Component: TestTextarea
});

const actionTypePrefix = 'TEST-TEXTAREA';
const path = 'component';
const root = 'test';

test('should dispatch change event on change', () => {
    const store = createStore({
        [root]: {
            [path]: textareaState()
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestTextareaContainer
                actionTypePrefix={ actionTypePrefix }
                customActionMetadata={ { index: 1 } }
                path={ path }
                root={ root } />
        </Provider>
    );

    provider.find(TestTextareaContainer).simulate('change', { target: { value: 'a' } });

    const actions = store.getState().actions;

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(textareaChange({
        custom: { index: 1 },
        path,
        root,
        type: changeType(actionTypePrefix),
        rawValue: 'a',
        value: 'a'
    }));
});

test('should read value from store', () => {
    const store = createStore({
        [root]: {
            [path]: textareaState({ rawValue: 'a', value: 'a' })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestTextareaContainer
                actionTypePrefix={ actionTypePrefix }
                path={ path }
                root={ root } />
        </Provider>
    );

    expect(provider.find(TestTextarea).prop('value')).toBe('a');
});
