import React from 'react';
import purebem from 'purebem';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import Textbox from './Textbox';
import TextboxContainer, { changeType } from './TextboxContainer';

import textboxChange from './redux/textboxChange';
import textboxTextState from './redux/textboxTextState';

import createStore from '../../test-utils/createStore';


const TestTextbox = Textbox({
    block: purebem.of('test-textbox'),
    displayName: 'TestTextbox'
});

const TestTextboxContainer = TextboxContainer({
    Component: TestTextbox
});

const actionTypePrefix = 'TEST-TEXTBOX';
const path = 'component';
const root = 'test';

test('should dispatch change event on change', () => {
    const store = createStore({
        [root]: {
            [path]: textboxTextState()
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestTextboxContainer
                actionTypePrefix={ actionTypePrefix }
                customActionMetadata={ { index: 1 } }
                path={ path }
                root={ root } />
        </Provider>
    );

    provider.find(TestTextboxContainer).simulate('change', { target: { value: 'a' } });

    const actions = store.getState().actions;

    expect(actions.length).toBe(1);
    expect(actions[0]).toEqual(textboxChange({
        custom: { index: 1 },
        path,
        root,
        type: changeType(actionTypePrefix),
        rawValue: 'a',
        value: 'a'
    }));
});

test('should read value from store', () => {
    const store = createStore({
        [root]: {
            [path]: textboxTextState({ rawValue: 'a', value: 'a' })
        }
    });

    const provider = mount(
        <Provider store={ store }>
            <TestTextboxContainer
                actionTypePrefix={ actionTypePrefix }
                path={ path }
                root={ root } />
        </Provider>
    );

    expect(provider.find(TestTextbox).prop('value')).toBe('a');
});
