import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';
import purebem from 'purebem';

import isNullOrEmpty from '../../utils/isNullOrEmpty';
import noop from '../../utils/noop';


export default ({
    block,
    displayName
}= {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            icon: PropTypes.string,
            onClick: PropTypes.func,
            // ---
            children: PropTypes.node,
            disabled: PropTypes.bool
        },

        getDefaultProps() {
            return {
                disabled: false,
                onClick: noop
            };
        },

        onClick(evt) {
            if (this.props.disabled) {
                return;
            }

            this.props.onClick(evt);
        },

        symbolRefCallback(component) {
            if (component === null) {
                return null;
            }

            const input = ReactDOM.findDOMNode(this.input);
            const padding = parseFloat(window.getComputedStyle(component)['padding-right'], 10);

            input.style.paddingRight = component.getBoundingClientRect().width + (Number.isNaN(padding) ? 0 : padding) + 'px';
        },

        render() {
            const { children, disabled, icon, onClick } = this.props;

            const node = React.Children.only(children);
            const clone = React.cloneElement(node, { ref: (input) => this.input = input });

            const clickable = !isNullOrEmpty(onClick) && onClick !== noop;

            return (
                <div className={ block({ clickable, disabled }) }>
                    { clone }
                    <i
                        className={ purebem.many(block('symbol'), `bi-${icon}`) }
                        onClick={ this.onClick }
                        ref={ this.symbolRefCallback } />
                </div>
            );
        }

    });

};
