import Immutable from 'seamless-immutable';

import handleFoundationAction from './handleFoundationAction';


test('ignore non-relevant action', () => {
    const state = Immutable({ component: { value: 1 } });
    const handle = handleFoundationAction('test');
    const result = handle(state, { type: 'NON-RELEVANT-ACTION' });

    expect(result).toEqual(state);
});

test('ignore non-relevant foundation action', () => {
    const state = Immutable({ component: { obj: { value: 1 } } });
    const handle = handleFoundationAction('test');
    const result = handle(state, {
        type: 'NON-RELEVANT-ACTION',
        meta: {
            root: 'component',
            path: 'obj',
            foundation: 'non-relevant-foundation-component'
        },
        payload: {
            value: 2
        }
    });

    expect(result).toEqual(state);
});

test('update state for relevant action', () => {
    const foundation = 'test';

    const state = Immutable({ component: { obj: { value: 1 } } });
    const handle = handleFoundationAction(foundation);
    const result = handle(state, {
        type: 'RELEVANT-ACTION',
        meta: {
            root: 'component',
            path: 'obj',
            foundation
        },
        payload: {
            value: 2
        }
    });

    expect(result).toEqual({
        component: {
            obj: {
                value: 2
            }
        }
    });
});
